@@ -1,10 +1,14 @@
 #!/usr/bin/env python3
+import sys
+
+if (sys.version_info.major, sys.version_info.minor) < (3, 7):
+    print("Python 3.7 or higher is needed", file=sys.stderr)
+    sys.exit(-1)
 
 import os
 import argparse
-import dataclasses
 import json
-import sys
+import dataclasses
 import tempfile
 import shutil
 import subprocess
@@ -14,7 +18,8 @@ from urllib.parse import urlparse
 from pathlib import Path
 from typing import List, Dict, Tuple, Union, Optional
 
-VERSION = "0.1.9"
+
+VERSION = "0.2.2"
 
 GIT_REPOSITORY = "https://github.com/csound-plugins/risset-data"
 
@@ -72,6 +77,11 @@ class Plugin:
     long_description: str = ""
     doc_folder: str = ""
 
+    def asdict(self) -> dict:
+        d = dataclasses.asdict(self)
+        d['manifest_path'] = str(self.manifest_path)
+        return d
+
 
 @dataclasses.dataclass
 class Opcode:
@@ -143,6 +153,14 @@ def _get_binary(binary) -> Optional[str]:
     return path if path else None
 
 
+def _get_git_binary() -> str:
+    path = shutil.which("git")
+    if not path:
+        raise RuntimeError("git binary not found")
+    assert os.path.exists(path)
+    return path
+
+
 def _git_clone(repo:str, destination:Path) -> None:
     """
     Clone the given repository to the destination.
@@ -153,7 +171,10 @@ def _git_clone(repo:str, destination:Path) -> None:
         raise ValueError("Destination should be an absolute path")
     if destination.exists():
         raise OSError("Destination path already exists, can't clone git repository")
-    gitbin = _get_binary("git")
+    gitbin = _get_git_binary()
+    if gitbin is None:
+        raise RuntimeError("git binary could not be found")
+
     parent = destination.parent
     if not parent.exists():
         parent.mkdir(parents=True, exist_ok=True)
@@ -172,7 +193,7 @@ def _git_update(repopath:Path) -> None:
     debug(f"Updating git repository: {repopath}")
     if not repopath.exists():
         raise OSError(f"Can't find path to git repository {repopath}")
-    gitbin = _get_binary("git")
+    gitbin = _get_git_binary()
     cwd = os.path.abspath(os.path.curdir)
     os.chdir(str(repopath))
     if SETTINGS['debug']:
@@ -734,18 +755,18 @@ class PluginsIndex:
 
     def get_plugin_dll(self, plugin:Plugin) -> Union[Path, ErrorMsg]:
         """
-        If the dll is a local file, just returns the absolute path,
-        otherwise it downloads the binary to a temporary file and
-        returns that path
+        Returns the path to the binary as defined in the manifest
         """
         binary_definition = plugin.binaries.get(self.platform)
         if not binary_definition:
             defined_platforms = ", ".join(plugin.binaries.keys())
-            error = ErrorMsg(f"No binary defined for platform {self.platform}."
-                             f" Available platforms for {plugin.name}: {defined_platforms}")
+            return ErrorMsg(f"No binary defined for platform {self.platform}."
+                            f" Available platforms for {plugin.name}: {defined_platforms}")
+
         # The manifest defines a path. If it is relative, it is relative to the
         # manifest itself.
-        path = resolve_path(binary_definition.url, Path(plugin.source).parent.as_posix())
+        path = resolve_path(binary_definition.url, Path(plugin.manifest_path).parent.as_posix())
+        debug(f"get_plugin_dll: resolved path = {str(path)}")
         if not path.exists():
             return ErrorMsg(f"Binary not found. Given path was: {str(path)}")
         return path
@@ -788,13 +809,13 @@ class PluginsIndex:
         manifests_path = self.get_installed_manifests_path()
         if not manifests_path.exists():
             manifests_path.mkdir(parents=True)
-        manifest = dataclasses.asdict(plugin)
+        manifest = plugin.asdict()
         manifest['build_platform'] = plugin.binaries[self.platform].build_platform
         manifest_path = manifests_path / f"{plugin.name}.json"
         try:
             manifest_json = json.dumps(manifest, indent=True)
         except Exception as e:
-            errormsg(str(e))
+            errormsg("install_plugin: error saving manifest: " + str(e))
             return ErrorMsg("Error when dumping manifest to json")
 
         with open(manifest_path.as_posix(), "w") as f:
@@ -818,9 +839,8 @@ class PluginsIndex:
         """
         opcodes = []
         for plugin in self.plugins:
-            for opcode in plugin.opcodes:
-                opcode = Opcode(name=opcode, plugin=plugin.name)
-                opcodes.append(opcode)
+            for opcodename in plugin.opcodes:
+                opcodes.append(Opcode(name=opcodename, plugin=plugin.name))
         return opcodes
 
     def expand_opcode_glob(self, pattern:str) -> List[Opcode]:
@@ -1191,10 +1211,10 @@ def cmd_man(plugins_index:PluginsIndex, args) -> bool:
         if matched:
             opcodes.extend(matched)
     if not opcodes:
-        debug(f"man: no opcodes found")
+        debug("man: no opcodes found")
         return False
     for opcode in opcodes:
-        debug(f"man: processing opcode {opcode}")
+        debug("man: processing opcode ", opcode)
         path = plugins_index.find_manpage(opcode.plugin, opcode.name, markdown=args.markdown)
         if not path:
             errormsg(f"No manpage for opcode {opcode.name}")
@@ -1220,7 +1240,7 @@ def add_flag(parser, flag, help=""):
 def main():
     # Preliminary checks
     if sys.platform not in ("linux", "darwin", "win32"):
-        errormsg("Platform not supported: {sys.platform}")
+        errormsg(f"Platform not supported: {sys.platform}")
         sys.exit(-1)
 
     if _get_binary("git") is None:
@@ -1236,14 +1256,14 @@ def main():
 
     # List command
     list_group = subparsers.add_parser('list', help="List packages")
-    add_flag(list_group, "--json", help="Outputs list as json")
+    add_flag(list_group, "--json", help="Outputs list as json (not implemented yet)")
     add_flag(list_group, "--all", "List all plugins, even those without a binary for the current platform")
-    list_group.add_argument("-o", "--outfile", help="Outputs to a file")
+    list_group.add_argument("-o", "--outfile", help="Outputs to a file (not implemented yet)")
     list_group.set_defaults(func=cmd_list)
 
     # Install command
     install_group = subparsers.add_parser("install", help="Install a package")
-    add_flag(install_group, "--user", help="Install in user folder")
+    add_flag(install_group, "--user", help="Install in user folder (not supported)")
     add_flag(install_group, "--force", help="Force install/reinstall")
     install_group.add_argument("plugins", nargs="+",
                                help="Name of the plugin/plugins to install. "
